﻿using System;
using System.ServiceModel;
using System.Transactions;
using System.Collections.Generic;
using Firma.Anwendung.BusinessEntities;

namespace Firma.Anwendung.Services
{
    [ServiceContract(Namespace = "Firma.Anwendung.Services", Name = "IOrderService", SessionMode = SessionMode.NotAllowed )]
    public interface IOrderService
    {
        #region Service Specific
        /// <summary>
        /// Get information (e.g. name, version) about the service
        /// </summary>
        /// <returns>Information about the service</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        ServiceInfo GetServiceInfo();
        #endregion

        #region Order
        /// <summary>
        /// Creates a new Order Object
        /// </summary>
        /// <param name="order"></param>
        /// <returns>OrderID of the new row as int</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        //[OperationBehaviorAttribute(ReleaseInstanceMode = ReleaseInstanceMode.AfterCall)]
        int CreateOrder(Order order);

        /// <summary>
        /// Updates the customer object
        /// </summary>
        /// <param name="customer">The customer object to be updated</param>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        void UpdateOrder(Order new_order, Order old_order);

        /// <summary>
        /// Deletes the customer object
        /// </summary>
        /// <param name="customerId">The application id of the customer</param>
        [OperationContract]
        [FaultContract(typeof(ServiceValidationFault))]
        [FaultContract(typeof(ServiceFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        void DeleteOrder(int orderId);

        /// <summary>
        /// Returns the customer object or null
        /// </summary>
        /// <param name="customerId">The application id of the customer</param>
        /// <returns>Customer object or null if no customer object exist</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Order GetOrderById(int orderId);

        /// <summary>
        /// Returns an array of customer objects
        /// </summary>
        /// <returns>Array of customer objects or empty array if no customer objects are defined</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Order[] GetOrdersforCustomer(string customerID);

        /// <summary>
        /// Returns an array of OrderDetail Objects
        /// </summary>
        /// <returns>Array of OrderDetail objects or empty array if no OrderDetail objects are defined</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        OrderDetail[] GetOrderDetailsByOrderID(int orderID);

        /// <summary>
        /// Returns an array of OrderDetailProductDetail Objects
        /// </summary>
        /// <returns>Array of OrderDetailProductDetail objects or empty array if no OrderDetailProductDetail objects are defined</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        OrderDetailProductDetail[] GetOrderandProductDetailsByOrderID(int orderID);
        
        /// <summary>
        /// Creates a new OrderDetail Object for Order with orderID
        /// </summary>
        /// <param name="order"></param>
        /// <returns>OrderDetailID of the new row as int</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        void CreateOrderDetail(OrderDetail orderDetail);

        /// <summary>
        /// Updates a OrderDetail Object for Order with orderID
        /// </summary>
        /// <param name="new_orderDetail"></param>
        /// <param name="old_orderDetail"></param>
        /// <returns>OrderDetailID of the new row as int</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        void UpdateOrderDetail(OrderDetail new_orderDetail, OrderDetail old_orderDetail);

        /// <summary>
        /// Creates a new Order Object and OrderDeatil(s) for that Order
        /// </summary>
        /// <param name="order></param>
        /// <param name="orderDetails[]"></param>
        /// <returns>number of OrderDetails created</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        int CreateOrderWithOrderDetails(Order order, OrderDetail[] orderDetails);

        #endregion

    }
}

