﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Data.Common;
using System.Reflection;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data;

namespace Firma.Anwendung.Utility
{
    public class ETUtility
    {
        /// <summary>
        /// The Logger.
        /// </summary>
        private static readonly LogUtility m_oLogUtility = new LogUtility(typeof(ETUtility));

        /// <summary>
        /// Get a formates String representaion of the object
        /// </summary>
        /// <param name="o">object to be converted</param>
        /// <param name="formatString">the format string (null = "{0} = {1}\n"))</param>
        /// <returns></returns>
        public static String ToFormatString(object o, string formatString)
        {
            StringBuilder sb = new StringBuilder();

            try
            {
                if (o == null)
                {
                    return "null";
                }
                else
                {
                    sb.AppendLine("\n>>>>>>> " + o.GetType().FullName + " >>>>>>>>>");
                    if (o is IEnumerable)
                    {
                        int index = 0;
                        foreach (object obj in o as IEnumerable)
                        {
                            sb.Append("Element #:" + index++);
                            sb.Append(ToFormatString(obj, formatString));
                        }
                        sb.Append("#Elements:" + index);
                        return sb.ToString();
                    }


                    // Use reflection to get all properties
                    PropertyInfo[] properties = o.GetType().GetProperties();

                    // Set default fomrat, if no format is specified
                    if (formatString == null)
                    {
                        formatString = "{0} = {1}\n";
                    }
                    // Add property states to StringBuilder
                    foreach (PropertyInfo property in properties)
                    {
                        Type propType = property.PropertyType;
                        string value;

                        if (propType == typeof(int) ||
                            propType == typeof(string) ||
                            propType == typeof(bool) ||
                            propType == typeof(float) ||
                            propType == typeof(double) ||
                            propType == typeof(UInt16) ||
                            propType == typeof(UInt32) ||
                            propType == typeof(UInt64) ||
                            propType == typeof(DateTime))
                        {
                            object propVal = property.GetValue(o, null);
                            if (propVal == null)
                            {
                                value = "null";
                            }
                            else
                            {
                                value = string.Format("\"{0}\"", propVal);
                            }
                        }
                        else if (propType.IsEnum)
                        {
                            object propVal = property.GetValue(o, null);
                            value = propVal.ToString();
                        }
                        else
                        {
                            object propVal = property.GetValue(o, null);

                            if (propVal == null)
                            {
                                value = "null";
                            }
                            else
                            {
                                value = string.Format(
                                    "(object of type {0}) \"{1}\"",
                                    propType,
                                    propVal
                                    );
                            }
                        }

                        sb.AppendFormat(formatString, property.Name, value);
                    }
                }
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, oException.Message, oException);
            }
            sb.AppendLine("<<<<<<<<<<<<<<<<<<<<<<<<<<");
            // mask {} braces to avoid String.Format error
            return MaskFormatBraces(sb.ToString());
        }

        /// <summary>
        /// Get info about the called command and parameters
        /// </summary>
        /// <param name="command">The command</param>
        /// <returns>A string containing infos about the called command and parameters</returns>
        public static String GetDbCommandInfo(DbCommand command)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.AppendLine("Calling command " + command.CommandText + " with parameters:");
            if (command.Parameters.Count == 0)
            {
                strBuilder.AppendLine("<NONE>");
            }
            else
            {
                foreach (DbParameter dbParameter in command.Parameters)
                {
                    strBuilder.AppendLine("Parameter " + dbParameter.ParameterName + ":" + dbParameter.Value);
                }
            }
            return strBuilder.ToString();
        }
        /// <summary>
        /// Execute a non query
        /// </summary>
        /// <param name="database">Database</param>
        /// <param name="command">command</param>
        /// <returns>number of rows affected</returns>
        public static int ExecuteNonQuery(Database database, DbCommand command)
        {
            if (m_oLogUtility.IsDebugEnabled)
            {
                m_oLogUtility.TraceLog(LogLevel.Debug, GetDbCommandInfo(command));
            }
            int result;
            TimeLogger.Instance(typeof(ETUtility)).Start();
            try
            {
                result = database.ExecuteNonQuery(command);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceLog(LogLevel.Error, GetDbCommandInfo(command));
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                TimeLogger.Instance(typeof(ETUtility)).Stop();
                TimeLogger.Instance(typeof(ETUtility)).Log(command.CommandText);
            }
            return result;
        }
        /// <summary>
        /// Execute a query
        /// </summary>
        /// <param name="database">Database</param>
        /// <param name="command">command</param>
        /// <returns>number of rows affected</returns>
        public static IDataReader ExecuteReader(Database database, DbCommand command)
        {
            if (m_oLogUtility.IsDebugEnabled)
            {
                m_oLogUtility.TraceLog(LogLevel.Debug, GetDbCommandInfo(command));
            }
            IDataReader dataReader;
            TimeLogger.Instance(typeof(ETUtility)).Start();
            try
            {
                dataReader = database.ExecuteReader(command);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceLog(LogLevel.Error, GetDbCommandInfo(command));
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                TimeLogger.Instance(typeof(ETUtility)).Stop();
                TimeLogger.Instance(typeof(ETUtility)).Log(command.CommandText);
            }
            return dataReader;
        }
        /// <summary>
        /// Retrieves default Database object
        /// <param name="configFilename">The configFilename</param>
        /// </summary>
        /// <returns>default Database object</returns>
        public static Database GetDefaultDatabase(String configFilename)
        {
            Database oDatabase;
            if (m_oLogUtility.IsDebugEnabled)
            {
                m_oLogUtility.TraceLog(LogLevel.Debug, "GetDefaultDatabase({0})", configFilename);
            }
            try
            {
                if (m_oLogUtility.IsDebugEnabled)
                {
                    m_oLogUtility.TraceLog(LogLevel.Debug, "Try to establish database");
                }
                FileConfigurationSource dataSource = new FileConfigurationSource(configFilename);

                if (m_oLogUtility.IsDebugEnabled)
                {
                    m_oLogUtility.TraceLog(LogLevel.Debug, "Using Configfile " + configFilename);
                }
                DatabaseProviderFactory dbFactory = new DatabaseProviderFactory(dataSource);

                oDatabase = dbFactory.CreateDefault();
                if (m_oLogUtility.IsDebugEnabled)
                {
                    m_oLogUtility.TraceLog(LogLevel.Debug, "Using Database " + oDatabase.ConnectionStringWithoutCredentials);
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, "Error while configuring database :", ex);
                throw;
            }
            return oDatabase;
        }

        /// <summary>
        /// Get the root path for the service.
        /// </summary>
        /// <returns>The root path for the service.</returns>
        public static String GetServiceRootPath()
        {
            if (m_oLogUtility.IsDebugEnabled)
            {
                m_oLogUtility.TraceLog(LogLevel.Debug, "GetServiceRootPath()");
            }
            string rootPath;
            if (ConfigurationManager.AppSettings["ServiceConfigDir"] != null)
            {
                rootPath = AppDomain.CurrentDomain.BaseDirectory + "\\" + ConfigurationManager.AppSettings["ServiceConfigDir"];
            }
            else
            {
                rootPath = AppDomain.CurrentDomain.BaseDirectory + "\\";
            }
            if (m_oLogUtility.IsDebugEnabled)
            {
                m_oLogUtility.TraceLog(LogLevel.Debug, "GetServiceRootPath() rootPath:'{0}'", rootPath);
            }
            return rootPath;
        }

        /// <summary>
        /// Mask format braces in strings.
        /// </summary>
        /// <param name="str">The string containing format braces.</param>
        /// <returns>The string, where the format braces are masked.</returns>
        public static string MaskFormatBraces(String str)
        {
            if (str != null)
            {
                str = str.Replace("{", "{{");
                str = str.Replace("}", "}}");
            }
            return str;
        }
    }

}
