﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Security.Principal;
using System.ServiceModel;
using System.ServiceModel.Configuration;
using System.Threading;
using System.Web.Security;
using Firma.Anwendung.Services;
using Firma.Anwendung.Utility;

namespace Firma.Anwendung.Hosting
{
    /// <summary>
    /// Platform
    /// </summary>
    public enum Platform
    {
        /// <summary>
        /// Platform - Console
        /// </summary>
        Console,

        /// <summary>
        /// Platform - WindowsService
        /// </summary>
        WindowsService,

        /// <summary>
        /// ServiceBus - .Net ServiceBus in Azure
        /// </summary>
        ServiceBus,

        /// <summary>
        /// Platform - Unknown
        /// </summary>
        Unknown
    }

    /// <summary>
    /// ETHostingLibrary
    /// </summary>
    public class ETHostingLibrary
    {
        private static readonly LogUtility m_oLogUtility = new LogUtility(typeof(ETHostingLibrary));
        private Platform m_ePlatform = Platform.Unknown;
        private List<HostingItem> m_oHostingList = new List<HostingItem>();

        private class HostingItem
        {
            private ServiceHost m_oServiceHost = null;
            private Type m_oServiceType = null;

            public HostingItem(ServiceHost oServiceHost, Type oServiceType)
            {
                m_oServiceHost = oServiceHost;
                m_oServiceType = oServiceType;
            }

            public ServiceHost ServiceHost
            {
                get { return m_oServiceHost; }
            }

            public Type ServiceType
            {
                get { return m_oServiceType; }
            }
        }

        /// <summary>
        /// ETHostingLibrary
        /// </summary>
        /// <param name="ePlatform"></param>
        public ETHostingLibrary(Platform ePlatform)
        {
            m_ePlatform = ePlatform;
        }

        /// <summary>
        /// OutputMessage
        /// </summary>
        /// <param name="eLogLevel"></param>
        /// <param name="strMessage"></param>
        private void OutputMessage(LogLevel eLogLevel, string strMessage)
        {
            m_oLogUtility.TraceLog(eLogLevel, strMessage);
            if (m_ePlatform == Platform.Console)
            {
                if (eLogLevel != LogLevel.Info)
                {
                    Console.ForegroundColor = ConsoleColor.Red;
                }
                Console.WriteLine(strMessage);

                Console.ForegroundColor = ConsoleColor.Green;
            }
        }

        /// <summary>
        /// CreateHost
        /// </summary>
        /// <param name="oType"></param>
        private void CreateHost(Type oType)
        {
            ServiceHost oServiceHost = new ServiceHost(oType);
            oServiceHost.Faulted += delegate(object sender, EventArgs e)
            {
                oServiceHost.Abort();
                OutputMessage(LogLevel.Info, "Event Faulted: " + oServiceHost.Description.Name);
            };

            oServiceHost.Open();
            m_oHostingList.Add(new HostingItem(oServiceHost, oType));

            OutputMessage(LogLevel.Info, oServiceHost.Description.Name + " is running.");
        }

        /// <summary>
        /// CheckServiceConfig
        /// </summary>
        /// <returns></returns>
        private bool CheckServiceConfig()
        {
            bool fResult = true;

            for (int i = 0; i < m_oHostingList.Count; i++)
            {
                try
                {
                    if (m_oHostingList[i].ServiceType != null)
                    {
                        Object oObject = Activator.CreateInstance(m_oHostingList[i].ServiceType);
                        if (oObject != null)
                        {
                            IETBaseService oIETBaseService = oObject as IETBaseService;
                            if (oIETBaseService != null)
                            {
                                oIETBaseService.CheckServiceConfiguration();
                                OutputMessage(LogLevel.Info, "Read Service Config: " + m_oHostingList[i].ServiceHost.Description.Name);
                            }
                        }
                    }
                }
                catch (Exception oException)
                {
                    fResult = false;

                    OutputMessage(LogLevel.Error, "Exception: " + m_oHostingList[i].ServiceHost.Description.Name + " - " + oException.ToString());

                    if (m_ePlatform == Platform.Console)
                    {
                        Console.ForegroundColor = ConsoleColor.Red;
                        Console.WriteLine("Exception: " + oException.ToString());
                        Console.ReadKey();
                    }

                    break;
                }
            }

            return fResult;
        }

        /// <summary>
        /// Open
        /// </summary>
        /// <returns></returns>
        public bool Open()
        {
            bool fResult = false;

            try
            {
                // Set Principal
                //AppDomain.CurrentDomain.SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);
                //GenericIdentity oGenericIdentity = new GenericIdentity(WindowsIdentity.GetCurrent().Name);
                //RolePrincipal oRolePrincipal = new RolePrincipal(oGenericIdentity);
                //Thread.CurrentPrincipal = oRolePrincipal;
                //OutputMessage(LogLevel.Info, "User: " + Thread.CurrentPrincipal.Identity.Name);

                // Start Services Business
                CreateHost(typeof(CustomerService));
                CreateHost(typeof(OrderService));
                CreateHost(typeof(ProductService));
                // End Services Business

                // Start Services Infrastructure
                // End Services Infrastructure

                // Start Services Reports
                // End Services Reports

                // Start Services Workflows
               // End Services Workflows

                fResult = CheckServiceConfig();
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, "Open", oException);

                if (m_ePlatform == Platform.Console)
                {
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine("Hosting is not running!\n");
                    Console.WriteLine("Exception: " + oException.ToString());
                    Console.ReadKey();

                    Close();
                }
            }

            return fResult;
        }

        /// <summary>
        /// Close
        /// </summary>
        public void Close()
        {
            for (int i = 0; i < m_oHostingList.Count; i++)
            {
                try
                {
                    m_oHostingList[i].ServiceHost.Close();
                    OutputMessage(LogLevel.Info, m_oHostingList[i].ServiceHost.Description.Name + " is closed.");
                }
                catch (Exception oException)
                {
                    OutputMessage(LogLevel.Error, "Exception: " + m_oHostingList[i].ServiceHost.Description.Name + " - " + oException.ToString());

                    if (m_ePlatform == Platform.Console)
                    {
                        Console.ForegroundColor = ConsoleColor.Red;
                        Console.WriteLine("Exception: " + oException.ToString());
                        Console.ReadKey();
                    }
                }
            }

            m_oHostingList.Clear();
        }
    }
}
