﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Channels;
using Firma.Anwendung.Services;

namespace Firma.Anwendung.Utility
{
    /// <summary>proxy class for all services</summary>
    [Serializable]
    public class ServiceProxy : IServiceProxy, IDisposable
    {
        [Serializable]
        private class ProxyData : object
        {
            private readonly ICommunicationObject m_oCommunicationObject = null;
            private readonly string m_strService = string.Empty;
            private readonly string m_strCallStack = string.Empty;

            /// <summary>
            /// ProxyData
            /// </summary>
            /// <param name="oCommunicationObject"></param>
            /// <param name="strService"></param>
            /// <param name="strCallStack"></param>
            public ProxyData(ICommunicationObject oCommunicationObject, string strService, string strCallStack)
            {
                m_oCommunicationObject = oCommunicationObject;
                m_strService = strService;
                m_strCallStack = strCallStack;
            }

            /// <summary>
            /// CommunicationObject
            /// </summary>
            public ICommunicationObject CommunicationObject
            {
                get
                {
                    return m_oCommunicationObject;
                }
            }

            /// <summary>
            /// Service
            /// </summary>
            public string Service
            {
                get
                {
                    return m_strService;
                }
            }

            /// <summary>
            /// CallStack
            /// </summary>
            public string CallStack
            {
                get
                {
                    return m_strCallStack;
                }
            }
        }

        private readonly List<ProxyData> m_oProxyList = new List<ProxyData>();
        private readonly LogUtility m_oLogUtility = new LogUtility(typeof(ServiceProxy));
        private readonly Dictionary<string, Object> m_oChannelFactoryDictionary = new Dictionary<string, Object>();
        private bool m_fDisposed = false;

        /// <summary>
        /// Constructor, creates the channel
        /// </summary>
        public ServiceProxy()
        {
            AppDomain.CurrentDomain.ProcessExit += new EventHandler(ProcessExit);
        }

        /// <summary>
        /// ProcessExit
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ProcessExit(object sender, EventArgs e)
        {
            m_oLogUtility.TraceLog(LogLevel.Info, "ProcessExit - close channel (proxy and factory).");
            Close();
        }

        /// <summary>Dispose, closes the channel</summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Dispose
        /// </summary>
        /// <param name="disposing"></param>
        private void Dispose(bool disposing)
        {
            if (!m_fDisposed)
            {
                Close();

                m_fDisposed = true;
            }
        }

        /// <summary>closes the channel and factory</summary>
        public void Close()
        {
            try
            {
                lock (m_oProxyList)
                {
                    foreach (ProxyData oProxyData in m_oProxyList)
                    {
                        if (oProxyData.CommunicationObject != null)
                        {
                            try
                            {
                                if (oProxyData.CommunicationObject.State == CommunicationState.Opened)
                                {
                                    m_oLogUtility.TraceLog(LogLevel.Warning, "Channel: " + oProxyData.Service + " not closed! [CallStack: " + oProxyData.CallStack + "]");
                                    oProxyData.CommunicationObject.Close();
                                }
                                else if (oProxyData.CommunicationObject.State == CommunicationState.Faulted)
                                {
                                    m_oLogUtility.TraceLog(LogLevel.Warning, "Channel: " + oProxyData.Service + " Status Faulted! [CallStack: " + oProxyData.CallStack + "]");
                                    oProxyData.CommunicationObject.Abort();
                                }
                            }
                            catch (Exception oException)
                            {
                                m_oLogUtility.TraceException(LogLevel.Warning, "Close - Proxy", oException);
                            }
                        }
                    }

                    m_oProxyList.Clear();
                }

                lock (m_oChannelFactoryDictionary)
                {
                    foreach (KeyValuePair<string, Object> oKeyValuePair in m_oChannelFactoryDictionary)
                    {
                        if (oKeyValuePair.Value != null)
                        {
                            ICommunicationObject oCommunicationObject = oKeyValuePair.Value as ICommunicationObject;
                            if (oCommunicationObject != null)
                            {
                                try
                                {
                                    if (oCommunicationObject.State == CommunicationState.Opened)
                                    {
                                        oCommunicationObject.Close();
                                    }
                                    else
                                    {
                                        oCommunicationObject.Abort();
                                    }
                                }
                                catch (Exception oException)
                                {
                                    m_oLogUtility.TraceException(LogLevel.Warning, "Close - Factory", oException);
                                }
                                finally
                                {
                                    m_oLogUtility.TraceLog(LogLevel.Info, "Channel: " + oKeyValuePair.Key + " closed.");
                                }
                            }
                        }
                    }

                    m_oChannelFactoryDictionary.Clear();
                }
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, oException.Message, oException);
            }
        }

        /// <summary>close the proxy</summary>
        public void CloseProxy<I>(I oInstance) where I : class
        {
            try
            {
                ICommunicationObject oProxyObject = oInstance as ICommunicationObject;
                if ((oProxyObject != null) && ((oProxyObject.State != CommunicationState.Closed) || (oProxyObject.State != CommunicationState.Closing)))
                {
                    if (oProxyObject.State != CommunicationState.Faulted)
                    {
                        oProxyObject.Close();
                    }
                    else
                    {
                        oProxyObject.Abort();
                    }
                }
            }
            catch (Exception oException)
            {
               m_oLogUtility.TraceException(LogLevel.Error, oException.Message, oException);
            }
        }

        /// <summary>
        /// CreateInstance
        /// </summary>
        /// <typeparam name="I"></typeparam>
        /// <param name="strService"></param>
        /// <returns></returns>
        private I CreateInstance<I>(string strService) where I : class
        {
            I oIProxy = null;

            try
            {
                lock (m_oChannelFactoryDictionary)
                {
                    if ((!m_oChannelFactoryDictionary.ContainsKey(strService)) ||
                        (((CommunicationObject)(m_oChannelFactoryDictionary[strService])).State != CommunicationState.Opened))
                    {
                        ChannelFactory<I> oChannelFactory = new ChannelFactory<I>(strService);
                        m_oChannelFactoryDictionary[strService] = oChannelFactory;
                        m_oLogUtility.TraceLog(LogLevel.Info, "Create Channel Factory: " + strService);
                    }
#if DEBUG
                    //// adds a custom behavior with a client message inspector (for debugging purposes only)
                    //if (!((ChannelFactory<I>)m_oChannelFactoryDictionary[strService]).Endpoint.Behaviors.Contains(typeof(MessageAnalysisBehavior)))
                    //{
                    //    ((ChannelFactory<I>)m_oChannelFactoryDictionary[strService]).Endpoint.Behaviors.Add(new MessageAnalysisBehavior());
                    //}
#endif
                    oIProxy = ((ChannelFactory<I>)m_oChannelFactoryDictionary[strService]).CreateChannel();
                    m_oLogUtility.TraceLog(LogLevel.Info, "Create Proxy: " + strService);
                }

                lock (m_oProxyList)
                {
                    string strCallStack = string.Empty;
#if DEBUG
                    StackTrace oStackTrace = new StackTrace();
                    strCallStack = oStackTrace.ToString();
#endif
                    ProxyData oProxyData = new ProxyData(oIProxy as ICommunicationObject, strService, strCallStack);
                    m_oProxyList.Add(oProxyData);
                }
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, oException.Message, oException);

                oIProxy = null;
            }

            return oIProxy;
        }

        /// <summary>get the proxy</summary>
        public ICustomerService CustomerService
        {
            get
            {
                return CreateInstance<ICustomerService>("CustomerService");
            }
        }


    }
}

